# Игра "Игра Баше"

import arcade
import arcade.gui
from arcade.gui import UIAnchorWidget, UILabel
from random import randint
from Match import Match

from threading import Timer


# размеры окна в пикселях:
WINDOW_WIDTH = 860
WINDOW_HEIGHT = 440

# заголовок окна:
WINDOW_TITLE = "Игра Баше"

# размеры картинок в пикселях:
MATCH_WIDTH = 11
MATCH_HEIGHT = 101


# КЛАСС ИГРЫ
class Game(arcade.Window):
    # КОНСТРУКТОР:
    def __init__(self, width, height, title):
        super().__init__(width, height, title, center_window=True)
        # игроки:
        self.PLAYER = 1
        self.COMPUTER = 2
        # счёт игры:
        self.result = [0] * 3

        # игрок, делающий ход:
        self.player = self.PLAYER
        # победитель:
        self.winner = None

        # число ходов:
        self.nMove = 0

        # мин. и макс. число спичек в кучке:
        self.MIN_MATCH = 15
        self.MAX_MATCH = 30

        # число спичек в кучке:
        self.numMatch = 0
        # макс. число спичек, которые можно взять за 1 раз:
        self.maxGet = 0
        # осталось спичек в кучке:
        self.restMatch = 0

        # число выбранных спичек:
        self.chose_match = 0

        # список спичек:
        self.matches = []

        # сообщение:
        self.message = '           '

        self.timer = None

        # состояние игры:
        self.isGameOver = True
        self.setup()


    # ПОДГОТОВКА
    def setup(self):
        # элементы управления:
        self.uimanager = arcade.gui.UIManager()
        self.uimanager.enable()

        # кнопка:
        button_newgame = arcade.gui.UITextureButton(texture=arcade.load_texture('images/ng_a.png'),
                                                 texture_hovered=arcade.load_texture('images/ng_b.png'),
                                                 texture_pressed=arcade.load_texture('images/ng_c.png'))
        button_newgame.on_click = self.on_button_newgame_click
        # кнопка:
        button_move = arcade.gui.UITextureButton(texture=arcade.load_texture('images/na_a.png'),
                                                 texture_hovered=arcade.load_texture('images/na_b.png'),
                                                 texture_pressed=arcade.load_texture('images/na_c.png'))
        button_move.on_click = self.on_button_move_click

        # метки -->
        self.label_result = UILabel(text=self.message,
                        font_size=36, bold=True, width=300,
                        text_color=(255, 0, 255, 160))

        self.label_message = UILabel(text=self.message,
                        font_size=40, bold=True, width=600,
                        text_color=arcade.color_from_hex_string('#017638'))

        self.uimanager.add(UIAnchorWidget(child=self.label_result, align_x=-0, align_y=-190))
        self.uimanager.add(UIAnchorWidget(child=self.label_message, align_x=0, align_y=-40))
        self.uimanager.add(UIAnchorWidget(child=button_move, align_x=-290, align_y=-190))
        self.uimanager.add(UIAnchorWidget(child=button_newgame, align_x=290, align_y=-190))

        # загружаем картинки -->
        # загружаем фоновую картинку:
        self.back = arcade.load_texture("images/back.png")
        # нормальная спичка:
        normalMatch = arcade.load_texture("images/red.png")
        # выбранная спичка:
        choseMatch = arcade.load_texture("images/violet.png")

        for i in range(self.MAX_MATCH):
            self.matches.append(Match(i * 25 + 20, 180,
                MATCH_WIDTH, MATCH_HEIGHT, normalMatch, choseMatch))

        # загружвем звуки -->
        # выбираем спичку:
        self.chose = arcade.load_sound("sounds/chose.wav")
        # ошибка:
        self.error = arcade.load_sound("sounds/error.wav")
        # победа:
        self.win = arcade.load_sound("sounds/win.wav")
        # нажимаем на кнопку:
        self.press = arcade.load_sound("sounds/button-press.mp3")

        # начинаем игру:
        self.newGame()


    # НАЧИНАЕМ НОВУЮ ИГРУ
    def newGame(self):
        self.message = ''

        # загадываем число спичек в кучке:
        self.numMatch = randint(self.MIN_MATCH, self.MAX_MATCH)
        #self.numMatch = 19
        #print(self.numMatch)

        # загадываем число спичек,
        # которые можно взять за 1 раз:
        self.maxGet = randint(1, self.numMatch // 5)
        if (self.maxGet < 3):
            self.maxGet = 3

        # осталось в кучке:
        self.restMatch = self.numMatch

        # ни одна спичка не выбрана:
        self.chose_match = 0

        for i in range(self.MAX_MATCH):
            self.matches[i].chose = False
            if (i < self.numMatch):
                self.matches[i].live = True
            else:
                self.matches[i].live = False

        # ход Игрока:
        self.player = self.PLAYER
        # победитель:
        self.winner = None

        # задержка хода Компьютера:
        self.pause = 0.400

        # игра началась:
        self.isGameOver = False


    # ОБНОВЛЯЕМ СЦЕНУ
    def on_draw(self):
        self.clear()
        xc = WINDOW_WIDTH / 2
        yc = WINDOW_HEIGHT / 2
        self.back.draw_scaled(xc, yc)

        self.draw_matches()
        self.drawInfo()
        self.uimanager.draw()


    # РИСУЕМ СПИЧКИ
    def draw_matches(self):
        # рисуем спички:
        for m in self.matches:
            m.draw()


    # ОБНОВЛЯЕМ ИНФОРМАЦИЮ
    def drawInfo(self):
        # правила игры:
        message = f"Число спичек в кучке равно {self.numMatch}. За один раз можно взять 1..{self.maxGet}."
        #text(u"", 20, 120)
        arcade.draw_text(message, 20, WINDOW_HEIGHT - 105,
                         arcade.color.BLUE,
                         font_size=16, bold=False)
        message = "Победит тот, кто возьмёт последнюю спичку."
        arcade.draw_text(message, 20, WINDOW_HEIGHT - 130,
                         arcade.color.BLUE,
                         font_size=16, bold=False)

        # число оставшихся спичек:
        if (self.restMatch):
            message = f" = {self.restMatch}"
            arcade.draw_text(message, WINDOW_WIDTH - 90, WINDOW_HEIGHT - 275,
                             arcade.color.RED,
                             font_size=22, bold=True)

        # печатаем текущего игрока:
        message = 'Ход '
        if (self.player == self.PLAYER):
            message += 'Игрока'
        else:
            message += 'Компьютера'
        arcade.draw_text(message, 20, WINDOW_HEIGHT - 190,
                         (0, 120, 0),
                         font_size=20, bold=False)

        # сообщение компьютера:
        arcade.draw_text(self.message, 20, WINDOW_HEIGHT - 345,
                         arcade.color_from_hex_string('#7D1F08'),
                         font_size=20, bold=False)

        # счёт игры:
        strs = f"Счёт  {self.result[self.PLAYER]} : {self.result[self.COMPUTER]}"
        self.label_result.text = strs

        # и сообщение компьютера:
        if self.winner == self.COMPUTER:
            self.label_message.text = 'Победил Компьютер'
        elif self.winner == self.PLAYER:
            self.label_message.text = 'Победил Игрок'
        else: self.label_message.text = ''


    # НАЖИМАЕМ КНОПКУ "НОВАЯ ИГРА"
    def on_button_newgame_click(self, event):
        self.press.play()
        self.newGame()


    # НАЖИМАЕМ КНОПКУ МЫШКИ -
    # ИГРОК ВЫБИРАЕТ СПИЧКИ
    def on_mouse_press(self, x: float, y: float, button: int, modifiers: int):
        # игра закончена:
        if self.isGameOver:
            return

        if self.player != self.PLAYER:
            return

        # ищем нажатую спичку:
        for i in range(self.MAX_MATCH):
            # нашли:
            if self.matches[i].live and\
                    self.matches[i].over(x, y):
                # ранее выбранная спичка
                # становится невыбранной:
                if (self.matches[i].chose):
                    self.matches[i].chose = False
                    self.chose_match -= 1
                # невыбранная спичка
                # становится выбранной:
                else:
                    self.matches[i].chose = True
                    self.chose_match += 1

                # Игрок выбрал больше спичек,
                # чем разрешают правила:
                if (self.chose_match > self.maxGet):
                    # звук ошибки:
                    self.error.play()
                # щелчок по спичке:
                else:
                    self.chose.play()

                # стираем сообщение:
                self.message = ''
                # цикл прекращаем -
                # одним кликом можно
                # выбрать только 1 спичку:
                break


    # ИГРОК БЕРЁТ СПИЧКИ
    def on_button_move_click(self, event):
        self.movePlayer()

    # ИГРОК БЕРЁТ СПИЧКИ
    def movePlayer(self):
        # игра уже закончена:
        if self.isGameOver:
            return
        if (self.player != self.PLAYER):
            return
        # Игрок выбрал слишком много спичек:
        if (self.chose_match > self.maxGet or self.chose_match == 0):
            # ошибка:
            self.error.play()
            return
        # звук нажатия на кнопку:
        self.press.play()

        # печатаем сообщение:
        self.message = f"Игрок взял: {self.chose_match}"
        self.drawInfo()

        # удаляем спички:
        t1 = Timer(1, self.deleteMatches, args=None, kwargs=None)
        t1.start()

        # передаём ход Компьютеру:
        t2 = Timer(2, self.moveComputer, args=None, kwargs=None)
        t2.start()


    # УДАЛЯЕМ ВЫБРАННЫЕ СПИЧКИ
    def deleteMatches(self):
        # осталось спичек:
        self.restMatch -= self.chose_match
        # удаляем выбранные спички:
        for i in range(self.MAX_MATCH):
            if (self.matches[i].chose):
                self.matches[i].live = False

        # выбранных спичек нет:
        self.chose_match = 0
        self.isWin()
        # ход Игрока:
        if not self.isGameOver:
            self.player = self.PLAYER


    # КОМПЬЮТЕР БЕРЁТ СПИЧКИ
    def moveComputer(self):
        if self.isGameOver:
            return
        #print("ход Компьютера")
        # ход Компьютера:
        self.player = self.COMPUTER

        # можно взять:
        ns = min(self.maxGet, self.restMatch)

        # компьютер может забрать все спички
        # и победить:
        if (ns == self.restMatch):
            ns = self.restMatch
        # иначе Компьютер делает
        # случайный ход:
        else:
            ns = randint(1, ns)

        # выбор сделан:
        self.chose_match = ns

        # Компьютер выбирает спички справа:
        for i in range(self.MAX_MATCH):
            n = self.MAX_MATCH - i - 1
            if (self.matches[n].live):
                self.matches[n].chose = True
                ns -= 1
                if (ns == 0):
                    break
        # печатаем сообщение:
        self.message = f"Компьютер взял: {self.chose_match}"
        t = Timer(2, self.deleteMatches, args=None, kwargs=None)
        t.start()


    # ПРОВЕРЯЕМ, НЕ ЗАКОНЧИЛАСЬ ЛИ ИГРА
    def isWin(self):
        if (self.isGameOver):
            return
        # если спички ещё остались,
        # то игра продолжается:
        if (self.restMatch > 0):
            return

        # победитель игры:
        self.winner = self.player

        # изменяем счёт:
        self.result[self.winner] += 1
        # стираем сообщение:
        self.message = ''

        # музыка для победителя:
        self.win.play()

        # игра закончена:
        self.isGameOver = True


# ГЛАВНАЯ ФУНКЦИЯ
def main():
    # создаём экземпляр игры:
    Game(WINDOW_WIDTH, WINDOW_HEIGHT, WINDOW_TITLE)
    # запускаем программу:
    arcade.run()
    # закрываем программу:
    arcade.close_window()


if __name__ == "__main__":
    main()
