# # Игра "Пузыри 2"

import arcade
import arcade.gui
from random import randint

# размеры окна:
WINDOW_WIDTH = 422
WINDOW_HEIGHT = 384
# заголовок окна:
WINDOW_TITLE = "Пузыри 2"
# цвет фона:
BACKGROUND_COLOR = (200, 200, 200)

# размеры поля в клетках:
FIELD_WIDTH = 10
FIELD_HEIGHT = 8
# размеры клеток в пикселях:
B_WIDTH = 42
B_HEIGHT = 42

# отступы клеток от краёв окна:
OFFSET_X = B_WIDTH // 2 + 1
OFFSET_Y = B_HEIGHT // 2 + 4


# КЛАСС ПУЗЫРЯ
class Bubble():
    def __init__(self, b0, b1):
        # пузырь не лопнут:
        self.live = True
        # размеры и координаты пузыря:
        self.w = B_WIDTH
        self.h = B_HEIGHT
        self.xc = 0
        self.yc = 0
        # картинки с пузырями:
        self.b0 = b0
        self.b1 = b1

    # РИСУЕМ ПУЗЫРЬ
    def draw(self):
        if self.live:
            self.b1.draw_scaled(self.xc, self.yc)
        else:
            self.b0.draw_scaled(self.xc, self.yc)

    # проверяем, находится ли заданная точка
    # на пузыре:
    def over(self, px, py):
        x = self.xc - OFFSET_X
        y = self.yc - OFFSET_Y
        return (x <= px <= x + self.w) and (y <= py <= y + self.h)


# КЛАСС ИГРЫ
class Game(arcade.Window):
    # КОНСТРУКТОР:
    def __init__(self, width, height, title):
        super().__init__(width, height, title, center_window=True)
        self.setup()

    # ПОДГОТОВКА
    def setup(self):
        # цвет фона окна:
        arcade.set_background_color(BACKGROUND_COLOR)

        # загружаем картинки с пузырями:
        b0 = arcade.load_texture("images/b0.png")
        b1 = arcade.load_texture("images/b1.png")

        # загружвем звук:
        self.buljk = arcade.load_sound("sounds/buljk.wav")

        # массив игрового поля:
        self.bubble = [[Bubble(b0, b1) for row in range(FIELD_HEIGHT)] for col in range(FIELD_WIDTH)]
        # создаём игровое поле:
        self.createField()

        # элементы управления:
        self.uimanager = arcade.gui.UIManager()
        self.uimanager.enable()
        self.box = arcade.gui.UIBoxLayout()
        # кнопка:
        self.button = arcade.gui.UITextureButton(texture=arcade.load_texture('images/button_a.png'),
                                                 texture_hovered=arcade.load_texture('images/button_b.png'),
                                                 texture_pressed=arcade.load_texture('images/button_c.png'))
        # функция, которая вызывается при нажатии на кнопку:
        self.button.on_click = self.on_buttonclick
        self.uimanager.add(arcade.gui.UIAnchorWidget(child=self.box))

        # начинаем игру:
        self.newGame()

    # НАЧИНАЕМ НОВУЮ ИГРУ
    def newGame(self, b=False):
        self.lopnuli = 0
        self.isGameOver = False
        for row in range(FIELD_HEIGHT):
            for col in range(FIELD_WIDTH):
                self.bubble[col][row].live = True
        if b:
            self.box.remove(self.button)

    # СОЗДАЁМ ИГРОВОЕ ПОЛЕ
    def createField(self):
        for row in range(FIELD_HEIGHT):
            for col in range(FIELD_WIDTH):
                x = col * B_WIDTH + OFFSET_X
                y = row * B_HEIGHT + B_HEIGHT + OFFSET_Y
                self.bubble[col][row].xc = x
                self.bubble[col][row].yc = y

    # РИСУЕМ ПУЗЫРИ
    def drawBubbles(self):
        for row in range(FIELD_HEIGHT):
            for col in range(FIELD_WIDTH):
                b = self.bubble[col][row]
                b.draw()


    # ОБНОВЛЯЕМ СЦЕНУ
    def on_draw(self):
        self.clear()
        self.drawBubbles()
        self.drawInfo()
        if self.isGameOver:
            self.uimanager.draw()


    # ОБНОВЛЯЕМ ИНФОРМАЦИЮ
    def drawInfo(self):
        # толщина линий:
        h = 38
        # красная полоса -->
        # начало линии:
        x_start = 5
        # длина линии:
        len = WINDOW_WIDTH-x_start*2
        # конец линии:
        x_end = len + x_start
        # отступ линии от нижнего края:
        y = 23
        arcade.draw_line(x_start, y, x_end, y,
                         arcade.color.RED, h)

        # зелёная полоса -->
        len *= self.lopnuli / (FIELD_WIDTH * FIELD_HEIGHT)
        x_end = len + x_start
        arcade.draw_line(x_start, y, x_end, y,
                         arcade.color.DARK_GREEN, h)

        # печатаем сообщение:
        message = f"Лопнули: {self.lopnuli}"
        arcade.draw_text(message, 140, 14,
                         arcade.color.YELLOW,
                         font_size=20, bold=True)



    # НАЖИМАЕМ КНОПКУ МЫШКИ
    def on_mouse_press(self, x, y, button, modifiers):
        # игра уже закончена:
        if self.isGameOver:
            return
        # ищем нажатый пузырь:
        for row in range(FIELD_HEIGHT):
            for col in range(FIELD_WIDTH):
                # нашли и он живой:
                if self.bubble[col][row].over(x, y) and \
                    self.bubble[col][row].live:
                    # звук:
                    arcade.play_sound(self.buljk)
                    # лопаем пузырь:
                    self.lopnuli += 1
                    self.bubble[col][row].live = False
                    # проверяем, не закончилась ли игра:
                    self.testGameOver()
                    return

    # НАЖИМАЕМ КНОПКУ
    def on_buttonclick(self, event):
        #print("Button is clicked")
        self.newGame(True)

    # ПРОВЕРЯЕМ, НЕ ЗАКОНЧИЛАСЬ ЛИ ИГРА
    def testGameOver(self):
        # все пузыри лопнули:
        if (self.lopnuli == FIELD_HEIGHT * FIELD_WIDTH):
        #if (self.lopnuli == 1): #FIELD_HEIGHT * FIELD_WIDTH):
            # игра закончена:
            self.isGameOver = True
            self.box.add(self.button)




# ГЛАВНАЯ ФУНКЦИЯ
def main():
    # создаём экземпляр игры:
    game = Game(WINDOW_WIDTH, WINDOW_HEIGHT, WINDOW_TITLE)
    # запускаем программу:
    arcade.run()
    # закрываем программу:
    arcade.close_window()


if __name__ == "__main__":
    main()
